from math import sqrt

from pyevspace import EVector
import unittest


# class holding the vector tests
class Test_evector(unittest.TestCase):

    v111 = EVector((1, 1, 1))
    v123 = EVector((1, 2, 3))
    v123m = EVector((-1, -2, -3))

    def test_vector_add(self):
        self.assertEqual(self.v111 + self.v123, EVector((2, 3, 4)))
        self.assertEqual(self.v111 + self.v123m, EVector((0, -1, -2)))

        with self.assertRaises(TypeError) as cm:
            self.v111 + 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 + 1.0
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 + 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_iadd(self):
        v = EVector((1, 1, 1))
        v += self.v123
        self.assertEqual(v, EVector((2, 3, 4)))
        v = EVector((-1, -2, -3))
        v += self.v123
        self.assertEqual(v, EVector((0, 0, 0)))

        with self.assertRaises(TypeError) as cm:
            self.v111 += 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 += 1.0
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 += 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_subtract(self):
        self.assertEqual(self.v123 - self.v111, EVector((0, 1, 2)))
        self.assertEqual(self.v111 - self.v123, EVector((0, -1, -2)))

        with self.assertRaises(TypeError) as cm:
            self.v111 - 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 - 1.0
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 - 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_isubtract(self):
        v = EVector((1, 1, 1))
        v -= self.v123
        self.assertEqual(v, EVector((0, -1, -2)))
        v = EVector((1, 1, 1))
        v -= self.v123m
        self.assertEqual(v, EVector((2, 3, 4)))

        with self.assertRaises(TypeError) as cm:
            self.v111 -= 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 -= 1.0
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 -= 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_multiply(self):
        self.assertEqual(self.v123 * 2, EVector((2, 4, 6)))
        self.assertEqual(self.v123m * 0.5, EVector((-0.5, -1, -1.5)))

        with self.assertRaises(TypeError) as cm:
            self.v111 * 'a'
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 * EVector()
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_imultiply(self):
        v = EVector((1, 2, 3))
        v *= 2
        self.assertEqual(v, EVector((2, 4, 6)))
        v = EVector((-1, -1, -1))
        v *= 0.5
        self.assertEqual(v, EVector((-0.5, -0.5, -0.5)))

        with self.assertRaises(TypeError) as cm:
            self.v111 *= 'a'
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 *= EVector()
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_divide(self):
        self.assertEqual(self.v123 / 1, self.v123)
        self.assertEqual(self.v123m / 0.1, EVector((-10, -20, -30)))

        with self.assertRaises(TypeError) as cm:
            self.v111 / 'a'
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 / EVector()
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_idivide(self):
        v = EVector((1, 2, 3))
        v /= 2
        self.assertEqual(v, EVector((0.5, 1, 1.5)))
        v = EVector((-1, -1, -1))
        v /= 0.5
        self.assertEqual(v, EVector((-2, -2, -2)))

        with self.assertRaises(TypeError) as cm:
            self.v111 /= 'a'
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111 /= EVector()
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_mag(self):
        v = EVector((3, 4, 0))
        self.assertEqual(v.mag(), 5)
        x = sqrt(1/3)
        v = EVector((x, x, x))
        self.assertEqual(v.mag(), 1)

        with self.assertRaises(TypeError) as cm:
            self.v111.mag(1)
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_mag2(self):
        self.assertEqual(self.v123.mag2(), 14)
        self.assertEqual(self.v123m.mag2(), 14)

        with self.assertRaises(TypeError) as cm:
            self.v111.mag2(1)
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_normalize(self):
        x = sqrt(1/3)
        v = EVector((x, x, x))
        v.normalize()
        self.assertEqual(v, EVector((x, x, x)))
        v = EVector((3, 4, 0))
        v.normalize()
        self.assertEqual(v, EVector((0.6, 0.8, 0)))

        with self.assertRaises(TypeError) as cm:
            self.v111.normalize(1)
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_sequence(self):
        self.assertEqual(len(self.v111), 3)
        self.assertEqual(self.v123[0], 1)
        self.assertEqual(self.v123[-1], 3)

        with self.assertRaises(IndexError) as cm:
            x = self.v111[3]
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            x = self.v111[-4]
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            x = self.v111['a']
        self.assertEqual(TypeError, type(cm.exception))

        v = EVector((1, 2, 3))
        v[0] = 5
        self.assertEqual(v[0], 5)
        v[-1] = 68
        self.assertEqual(v[2], 68)

        with self.assertRaises(IndexError) as cm:
            self.v111[3] = 5
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            self.v111[-4] = 5
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v111[0] = 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_compare(self):
        v = EVector((1, 2, 3))
        self.assertEqual(v, self.v123)
        self.assertNotEqual(v, self.v111)
        self.assertTrue(v == self.v123)
        self.assertTrue(v != self.v111)
        self.assertFalse(v == self.v111)
        self.assertFalse(v != self.v123)

    def test_iterable(self):
        self.assertIn(1, self.v123)
        self.assertNotIn(0, self.v123)


if __name__ == '__main__':
    unittest.main()



from pyevspace import EVector, EMatrix

"""Here is some example code for the arithmetic capabilities of vectors and matrices
of the pyevspace module. Running this file from the command line will print similar
code to the console as well."""

def example():
    # initializing
    vector1 = EVector((1, 2, 3))
    vector2 = EVector((4, 5, 6))
    matrix1 = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
    matrix2 = EMatrix((1, 2, 3), (1, 2, 3), (1, 2, 3))

    # addition and subtraction
    # both vectors and matrices can be added or subtracted to their own types.
    # each operation adds or subtracts component wise
    vectorAdd = vector1 + vector2
    matrixAdd = matrix1 + matrix2
    vectorSub = vector1 - vector2
    matrixSub = matrix1 - matrix2
    # vectors and matrices can only add their own type to themselves
    badVectorAdd = vector1 + matrix1  # raises TypeError
    badMatrixAdd = matrix1 + 1  # raises TypeError

    # negative
    # both types can be negated as well, which negates each component of the vector or matrix
    vectorNeg = -vector1  # vectorNeg == EVector((-1, -2, -3))

    # scalar multiplication and division
    # both vectors and matrices support scalar multiplication
    vectorMult = vector1 * 2
    matrixMult = matrix1 * 3.14
    # both types also support scalar division
    vectorDiv = vector1 / 10
    # the scalar must be a numeric type (i.e. a float or provide __float__() or __index__())

    # inplace operators
    # both types also support inplace operators for addition, subtraction and
    # scalar multiplication and division

    # matrix multiplication
    # the matrix class supports left-hand multiplication of matrices on
    # other matrices and vectors
    newMatrix = matrix1 * matrix2
    newVector = matrix1 * vector1
    # note the vectors must be on the right of the matrix
    badVector = vector1 * matrix1  # this will cause a type error

    # comparison
    # both types support equality and inequality operators for their respective types
    vecEq = vector1 == vector2
    vecNe = vector1 != vector2
    matEq = matrix1 == matrix2
    matNe = matrix1 != matrix2

    ignore = [vectorAdd, matrixAdd, vectorSub, matrixSub, badVectorAdd, badMatrixAdd, vectorNeg, vectorMult,
              matrixMult, vectorDiv, newMatrix, newVector, badVector, vecEq, vecNe, matEq, matNe]


if __name__ == '__main__':
    string = """
# initializing
>>> vector1 = EVector((1, 2, 3))
>>> vector2 = EVector((4, 5, 6))
>>> matrix1 = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
>>> matrix2 = EMatrix((1, 2, 3), (1, 2, 3), (1, 2, 3))

# addition and subtraction
# both vectors and matrices can be added or subtracted to their own types.
# each operation adds or subtracts component wise
>>> vectorAdd = vector1 + vector2
>>> matrixAdd = matrix1 + matrix2
>>> vectorSub = vector1 - vector2
>>> matrixSub = matrix1 - matrix2
# vectors and matrices can only add their own type to themselves
>>> badVectorAdd = vector1 + matrix1  # raises TypeError
>>> badMatrixAdd = matrix1 + 1  # raises TypeError

# negative
# both types can be negated as well, which negates each component of the vector or matrix
>>> vectorNeg = -vector1  # vectorNeg == EVector((-1, -2, -3))

# scalar multiplication and division
# both vectors and matrices support scalar multiplication
>>> vectorMult = vector1 * 2
>>> matrixMult = matrix1 * 3.14
# both types also support scalar division
>>> vectorDiv = vector1 / 10
# the scalar must be a numeric type (i.e. a float or provide __float__() or __index__())

# inplace operators
# both types also support inplace operators for addition, subtraction and
# scalar multiplication and division

# matrix multiplication
# the matrix class supports left-hand multiplication of matrices on
# other matrices and vectors
>>> newMatrix = matrix1 * matrix2
>>> newVector = matrix1 * vector1
# note the vectors must be on the right of the matrix
>>> badVector = vector1 * matrix1  # this will cause a type error

# comparison
# both types support equality and inequality operators for their respective types
>>> vecEq = vector1 == vector2
>>> vecNe = vector1 != vector2
>>> matEq = matrix1 == matrix2
>>> matNe = matrix1 != matrix2"""

    print(string)



import unittest
from vector_test import Test_evector
from matrix_test import Test_ematrix
from module_test import Test_evspace


def get_suite():

    testClasses = [Test_evector, Test_ematrix, Test_evspace]

    loader = unittest.TestLoader()

    suitesList = []
    for testClass in testClasses:
        print('loading ', testClass.__name__)
        suite = loader.loadTestsFromTestCase(testClass)
        suitesList.append(suite)

    print('suites loaded')
    bigSuite = unittest.TestSuite(suitesList)

    return bigSuite


def execute_tests():

    bigSuite = get_suite()

    runner = unittest.TextTestRunner()
    runner.run(bigSuite)


if __name__ == '__main__':
    execute_tests()



from math import pi

from pyevspace import *
import unittest


class Test_evspace(unittest.TestCase):

    v = EVector((1, 2, 3))
    v34 = EVector((3, 4, 0))
    m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))

    def test_dot(self):
        self.assertEqual(dot(self.v, self.v34), 11)
        self.assertEqual(dot(self.v, self.v), self.v.mag2())

        with self.assertRaises(TypeError) as cm:
            dot(self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            dot(self.v, self.v, self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            dot(self.v, 0)
        self.assertEqual(TypeError, type(cm.exception))

    def test_cross(self):
        self.assertEqual(cross(self.v, self.v34), EVector((-12, 9, -2)))
        self.assertEqual(cross(self.v34, self.v), EVector((12, -9, 2)))

        with self.assertRaises(TypeError) as cm:
            cross(self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            cross(self.v, self.v, self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            cross(self.v, 1.0)
        self.assertEqual(TypeError, type(cm.exception))

    def test_norm(self):
        self.assertEqual(norm(self.v34), EVector((0.6, 0.8, 0)))
        self.assertEqual(norm(self.v34 * -1), EVector((-0.6, -0.8, 0)))
        v = EVector((1, 2, 3))
        v.normalize()
        self.assertEqual(norm(self.v), v)

        with self.assertRaises(TypeError) as cm:
            norm()
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            norm(self.v, self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            norm(1)
        self.assertEqual(TypeError, type(cm.exception))

    def test_vang(self):
        self.assertAlmostEqual(vang(EVector((1, 1, 0)), EVector((1, 0, 0))), pi / 4, 6)
        self.assertAlmostEqual(vang(EVector((1, 0, 0)), EVector((0, 1, 0))), pi / 2, 6)

        with self.assertRaises(TypeError) as cm:
            vang(self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            vang(self.v, self.v, self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            vang(2)
        self.assertEqual(TypeError, type(cm.exception))

    def test_vxcl(self):
        self.assertEqual(vxcl(EVector((1, 1, 0)), EVector((0, 1, 0))), EVector((1, 0, 0)))
        self.assertEqual(vxcl(EVector((1, 2, 0)), EVector((0, 1, 0))), EVector((1, 0, 0)))

        with self.assertRaises(TypeError) as cm:
            vang(self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            vang(self.v, self.v, self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            vang(5)
        self.assertEqual(TypeError, type(cm.exception))

    def test_det(self):
        self.assertEqual(det(self.m), 0)
        m = EMatrix((2, 6, 4), (7, 3, 1), (8, 0, 0))
        self.assertEqual(det(m), -48)

        with self.assertRaises(TypeError) as cm:
            det()
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            det(self.v, self.v)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            det(7)
        self.assertEqual(TypeError, type(cm.exception))

    def test_transpose(self):
        ans = EMatrix((1, 4, 7), (2, 5, 8), (3, 6, 9))
        self.assertEqual(transpose(self.m), ans)

        with self.assertRaises(TypeError) as cm:
            transpose()
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            transpose(self.m, self.m)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            transpose(5)
        self.assertEqual(TypeError, type(cm.exception))


if __name__ == '__main__':
    unittest.main()



from pyevspace import *

"""Here is some code to show what module methods are provided by the module.
Running this file from the command line will output similar code."""

def example():
    # initialize
    vector1 = EVector((1, 2, 3))
    vector2 = EVector((4, 5, 6))
    matrix1 = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))

    # dot product   dot(EVector, EVector) -> float
    # the dot() method computes the dot product of two vectors
    dotProduct = dot(vector1, vector2)  # dotProduct = 32

    # cross product     cross(EVector, EVector) -> EVector
    # the cross() method computes the cross product of two vectors
    # using a right-handed coordinate system.
    crossVector = cross(vector1, vector2)
    # to get the cross product in a left-handed coordinate frame
    # use the vector negative to invert the direction
    crossLeft = -cross(vector1, vector2)

    # normal vector     norm(EVector) -> EVector
    # the norm() method returns a normalized vector of its argument
    normalVector = norm(vector1)  # normalVector has magnitude 1
    # the EVector class also has a class method to normalize a vector in place
    vector3 = EVector((1, 2, 3))
    vector3.normalize()  # vector3 has magnitude 1

    # vector angle      vang(EVector, EVector) -> float
    # the vang() method computes the shortest angle between two vectors. for
    # two vectors in a plane, there are two angles between them, which add to 2pi.
    # the vang() methods always returns the shortest one. ensuring the shortest angle
    # is the angle you may need is project dependent and out of the scope of this example
    vectorAngle = vang(vector1, vector2)

    # vector exclude    vxcl(EVector, EVector) -> EVector
    # the vxcl() method returns a vector equal to one vector with all components of another
    # removed from it. in essence, it returns the projection of one vector onto a plane with
    # the other vector as its normal vector.
    vectorExclude = vxcl(EVector((1, 2, 3)), EVector((0, 1, 0)))  # vectorExclude = (1, 0, 3)

    # determinate   det(EMatrix) -> float
    # the det() method computes the determinate of a matrix, useful for determining if
    # a matrix is invertible
    determinate = det(matrix1)

    # transpose     transpose(EMatrix) -> EMatrix
    # the transpose() method returns a new matrix that is the transpose of its argument
    trans = transpose(matrix1)

    ignore = [dotProduct, crossVector, crossLeft, normalVector, vectorAngle, vectorExclude, determinate, trans]


if __name__ == '__main__':
    string = """
# initialize
>>> vector1 = EVector((1, 2, 3))
>>> vector2 = EVector((4, 5, 6))
>>> matrix1 = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))

# dot product   dot(EVector, EVector) -> float
# the dot() method computes the dot product of two vectors
>>> dotProduct = dot(vector1, vector2)  # dotProduct = 32

# cross product     cross(EVector, EVector) -> EVector
# the cross() method computes the cross product of two vectors
# using a right-handed coordinate system.
>>> crossVector = cross(vector1, vector2)
# to get the cross product in a left-handed coordinate frame
# use the vector negative to invert the direction
>>> crossLeft = -cross(vector1, vector2)

# normal vector     norm(EVector) -> EVector
# the norm() method returns a normalized vector of its argument
>>> normalVector = norm(vector1)  # normalVector has magnitude 1
# the EVector class also has a class method to normalize a vector in place
>>> vector3 = EVector((1, 2, 3))
>>> vector3.normalize()  # vector3 has magnitude 1

# vector angle      vang(EVector, EVector) -> float
# the vang() method computes the shortest angle between two vectors. for
# two vectors in a plane, there are two angles between them, which add to 2pi.
# the vang() methods always returns the shortest one. ensuring the shortest angle
# is the angle you may need is project dependent and out of the scope of this example
>>> vectorAngle = vang(vector1, vector2)

# vector exclude    vxcl(EVector, EVector) -> EVector
# the vxcl() method returns a vector equal to one vector with all components of another
# removed from it. in essence, it returns the projection of one vector onto a plane with
# the other vector as its normal vector.
>>> vectorExclude = vxcl(EVector((1, 2, 3)), EVector((0, 1, 0)))  # vectorExclude = (1, 0, 3)

# determinate   det(EMatrix) -> float
# the det() method computes the determinate of a matrix, useful for determining if
# a matrix is invertible
>>> determinate = det(matrix1)

# transpose     transpose(EMatrix) -> EMatrix
# the transpose() method returns a new matrix that is the transpose of its argument
>>> trans = transpose(matrix1)"""

    print(string)



from setuptools import setup, Extension

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

ext_modules = [Extension(
    '_pyevspace',
    include_dirs=['pyevspace/include'],
    sources=['pyevspace/src/evspacemodule.c'],
)]

setup(name='pyevspace',
      version='0.0.9',
      author='Quinton Barnes',
      author_email='devqbizzle68@gmail.com',
      description='A Euclidean vector space module.',
      long_description=long_description,
      long_description_content_type='text/markdown',
      license='MIT',
      url='https://github.com/qbizzle68/pyevspace',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Science/Research',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: C',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: Implementation :: CPython',
          'Topic :: Scientific/Engineering :: Mathematics',
          'Topic :: Scientific/Engineering :: Physics',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      packages=['pyevspace', 'pyevspace.examples', 'pyevspace.tests'],
      ext_modules=ext_modules,
      )



from copy import copy

from pyevspace import EVector, EMatrix

'''Here is some example code for creating EVector and EMatrix instances. Running
this file from the command line will print similar code to the console as well.'''

def example():
    # create a vector
    vector = EVector((1, 2, 3))
    # vectors are created from any iterable object with a length of exactly 3
    ls = [7, 8, 9]
    vector2 = EVector(ls)
    # if creating a vector from values, they must be wrapped in an iterable
    # this raises a TypeError exception !!!
    vector3 = EVector(1, 2, 3)  # bad constructor call
    # values also must be a numeric type. if they are not a float type then
    # the constructor will invoke __float__() if available, then __index__()
    # if still needed. otherwise a TypeError exception will be raised.
    vector4 = EVector('a', [0, 2], range(2))  # all of these values would cause
    #                                           a TypeError exception

    # creating a matrix
    matrix = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
    # similarly to vectors, matrices are created from exactly 3 iterable objects, that each
    # have a length of exactly 3
    ls = [1, 2, 3]
    tp = (4, 5, 6)
    v = EVector((7, 8, 9))
    matrix2 = EMatrix(ls, tp, v)
    # similarly to a vector, a matrix values must be numeric, meaning they are a float type,
    # or they supply a __float__() or __index__() method (used in that order).

    # get / set components
    # both vectors and matrix elements can be accessed with the [] operator
    a = vector[0]  # returns 1
    b = vector2[-1]  # returns 9
    c = matrix[2, 1]  # return 8
    # both vector and matrix elements can be set with the [] operator
    vector[0] = 5
    vector2[-2] = 7
    matrix[1, 1] = 0
    # similar to construction, all vector and matrix elements must be numeric
    vector[2] = 'a'  # raises TypeError

    # vector as iterable
    # a vector also acts as an iterator and can be used anywhere an iterable can be used
    sumValue = 0
    for v in vector:
        sumValue += v
    # sum is now equal to vector[0] + vector[1] + vector[2]
    # the in operator can also be used to find if an element is in a vector
    result = 5 in vector  # result will be true if 5 is an element of the vector

    # copying
    # both types provide Pickle support by implementing __reduce__(). This allows
    # objects to be copied with the copy module.
    vectorCopy = copy(vector)
    matrixCopy = copy(matrix)

    ignore = [vector3, vector4, matrix2, a, b, c, result, vectorCopy, matrixCopy]


if __name__ == '__main__':
    string = """
# create a vector
>>> vector = EVector((1, 2, 3))
# vectors are created from any iterable object with a length of exactly 3
>>> ls = [7, 8, 9]
>>> vector2 = EVector(ls)
# if creating a vector from values, they must be wrapped in an iterable
# this raises a TypeError exception !!!
>>> vector3 = EVector(1, 2, 3)  # bad constructor call
# values also must be a numeric type. if they are not a float type then
# the constructor will invoke __float__() if available, then __index__()
# if still needed. otherwise a TypeError exception will be raised.
>>> vector4 = EVector('a', [0, 2], range(2))  # all of these values would cause
#                                               a TypeError exception

# creating a matrix
>>> matrix = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
# similarly to vectors, matrices are created from any iterable object with a length of
# exactly 3, whose values are any iterable object with a length of exactly 3
>>> ls = [1, 2, 3]
>>> tp = (4, 5, 6)
>>> v = EVector((7, 8, 9))
>>> matrix2 = EMatrix((ls, tp, v))
# similarly to a vector, a matrix values must be numeric, meaning they are a float type,
# or they supply a __float__() or __index__() method (used in that order).

# get / set components
# both vectors and matrix elements can be accessed with the [] operator
>>> x = vector[0]  # returns 1
>>> x = vector2[-1]  # returns 9
>>> x = matrix[2, 1]  # return 8
# both vector and matrix elements can be set with the [] operator
>>> vector[0] = 5
>>> vector2[-2] = 7
>>> matrix[1, 1] = 0
# similar to construction, all vector and matrix elements must be numeric
>>> vector[2] = 'a'  # raises TypeError

# vector as iterable
# a vector also acts as an iterator and can be used anywhere an iterable can be used
>>> sumValue = 0
>>> for v in vector:
>>>     sumValue += v
# sum is now equal to vector[0] + vector[1] + vector[2]
# the in operator can also be used to find if an element is in a vector
>>> result = 5 in vector  # result will be true if 5 is an element of the vector

# copying
# both types provide Pickle support by implementing __reduce__(). This allows
# objects to be copied with the copy module.
>>> vectorCopy = copy(vector)
>>> matrixCopy = copy(matrix)"""

    print(string)



from pyevspace import *
import unittest


class Test_ematrix(unittest.TestCase):

    mi = EMatrix((1, 0, 0), (0, 1, 0), (0, 0, 1))
    m123 = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
    m147 = EMatrix((1, 4, 7), (2, 5, 8), (3, 6, 9))
    v123 = EVector((1, 2, 3))
    m2 = EMatrix((2, 4, 6), (8, 10, 12), (14, 16, 18))

    def test_add(self):
        ans = EMatrix((2, 6, 10), (6, 10, 14), (10, 14, 18))
        self.assertEqual(self.m123 + self.m147, ans)

        with self.assertRaises(TypeError) as cm:
            self.v123 + 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v123 + 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_iadd(self):
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m += self.m147
        ans = EMatrix((2, 6, 10), (6, 10, 14), (10, 14, 18))
        self.assertEqual(self.m123 + self.m147, ans)

        with self.assertRaises(TypeError) as cm:
            self.v123 += 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v123 += 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_subtract(self):
        ans = EMatrix((0, -2, -4), (2, 0, -2), (4, 2, 0))
        self.assertEqual(self.m123 - self.m147, ans)

        with self.assertRaises(TypeError) as cm:
            self.v123 - 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v123 - 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_isubtract(self):
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m -= self.m147
        ans = EMatrix((0, -2, -4), (2, 0, -2), (4, 2, 0))
        self.assertEqual(m, ans)

        with self.assertRaises(TypeError) as cm:
            self.v123 += 1
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v123 += 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_scalar_multiply(self):
        # ans = EMatrix((2, 4, 6), (8, 10, 12), (14, 16, 18))
        self.assertEqual(self.m123 * 2, self.m2)
        self.assertEqual(self.m123 * 2.0, self.m2)

        with self.assertRaises(TypeError) as cm:
            self.m123 * range(5)
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.v123 * 'a'
        self.assertEqual(TypeError, type(cm.exception))

    def test_vector_multiply(self):
        self.assertEqual(self.m123 * self.v123, EVector((14, 32, 50)))

    def test_matrix_multiply(self):
        ans = EMatrix((14, 32, 50), (32, 77, 122), (50, 122, 194))
        self.assertEqual(self.m123 * self.m147, ans)

    def test_imultiply(self):
        # ans = EMatrix((2, 4, 6), (8, 10, 12), (14, 16, 18))
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m *= 2
        self.assertEqual(m, self.m2)
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m *= 2.0
        self.assertEqual(m, self.m2)

    def test_divide(self):
        ans = EMatrix((0.5, 1, 1.5), (2, 2.5, 3), (3.5, 4, 4.5))
        self.assertEqual(self.m123 / 2, ans)
        self.assertEqual(self.m123 / 2.0, ans)
        self.assertEqual(self.m123 / 0.5, self.m2)

    def test_idivide(self):
        ans = EMatrix((0.5, 1, 1.5), (2, 2.5, 3), (3.5, 4, 4.5))
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m /= 2
        self.assertEqual(m, ans)
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m /= 2.0
        self.assertEqual(m, ans)
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m /= 0.5
        self.assertEqual(m, self.m2)

    def test_get(self):
        self.assertEqual(self.m123[0, 0], 1)
        self.assertEqual(self.m123[1, 2], 6)

        with self.assertRaises(SystemError) as cm:
            x = self.m123[0]
        self.assertEqual(SystemError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            x = self.m123[1, 'a']
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            x = self.m123[1, 0, 2]
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            x = self.m123[3, 1]
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            x = self.m123[2, 5]
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            x = self.m123[0, -5]
        self.assertEqual(IndexError, type(cm.exception))

    def test_set(self):
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        m[0, 0] = 42
        self.assertEqual(m[0, 0], 42)

        with self.assertRaises(SystemError) as cm:
            self.m123[0] = 4
        self.assertEqual(SystemError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.m123[1, 'a'] = 2
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(TypeError) as cm:
            self.m123[1, 0, 2] = 5
        self.assertEqual(TypeError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            self.m123[3, 1] = 3
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            self.m123[2, 5] = 8
        self.assertEqual(IndexError, type(cm.exception))

        with self.assertRaises(IndexError) as cm:
            self.m123[0, -5] = 7
        self.assertEqual(IndexError, type(cm.exception))

    def test_compare(self):
        m = EMatrix((1, 2, 3), (4, 5, 6), (7, 8, 9))
        self.assertEqual(m, self.m123)
        self.assertNotEqual(m, self.m147)
        self.assertTrue(m == self.m123)
        self.assertFalse(m == self.m147)
        self.assertTrue(m != self.m147)
        self.assertFalse(m != self.m123)


if __name__ == '__main__':
    unittest.main()
